@@ -1,23 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2016  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *
- *  Copyright (C) 2008-2016  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -44,12 +29,12 @@
 	ofono_dbus_dict_append(p_dict, key, dbus_type, &value); \
 } while (0)
 
-static GSList *g_drivers = NULL;
-
 struct ofono_netmon {
 	const struct ofono_netmon_driver *driver;
 	DBusMessage *pending;
 	DBusMessage *reply;
+	DBusMessageIter iter;
+	DBusMessageIter arr;
 	void *driver_data;
 	struct ofono_atom *atom;
 	struct netmon_agent *agent;
@@ -69,6 +54,177 @@ static const char *cell_type_to_tech_name(enum ofono_netmon_cell_type type)
 	return NULL;
 }
 
+static void netmon_cell_info_dict_append(DBusMessageIter *dict,
+					va_list *arglist, int info_type)
+{
+	char *mcc;
+	char *mnc;
+	int intval;
+	enum ofono_netmon_info next_info_type = info_type;
+
+	while (next_info_type != OFONO_NETMON_INFO_INVALID) {
+		switch (next_info_type) {
+		case OFONO_NETMON_INFO_MCC:
+			mcc = va_arg(*arglist, char *);
+
+			if (mcc && strlen(mcc))
+				ofono_dbus_dict_append(dict,
+						"MobileCountryCode",
+						DBUS_TYPE_STRING, &mcc);
+			break;
+
+		case OFONO_NETMON_INFO_MNC:
+			mnc = va_arg(*arglist, char *);
+
+			if (mnc && strlen(mnc))
+				ofono_dbus_dict_append(dict,
+						"MobileNetworkCode",
+						DBUS_TYPE_STRING, &mnc);
+			break;
+
+		case OFONO_NETMON_INFO_LAC:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "LocationAreaCode",
+					intval, uint16_t, DBUS_TYPE_UINT16);
+			break;
+
+		case OFONO_NETMON_INFO_CI:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "CellId",
+					intval, uint32_t, DBUS_TYPE_UINT32);
+			break;
+
+		case OFONO_NETMON_INFO_ARFCN:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "ARFCN",
+					intval, uint16_t, DBUS_TYPE_UINT16);
+			break;
+
+		case OFONO_NETMON_INFO_BSIC:
+				intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "BSIC",
+					intval, uint8_t, DBUS_TYPE_BYTE);
+			break;
+
+		case OFONO_NETMON_INFO_RXLEV:
+				intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "ReceivedSignalStrength",
+					intval, uint8_t, DBUS_TYPE_BYTE);
+			break;
+
+		case OFONO_NETMON_INFO_TIMING_ADVANCE:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "TimingAdvance",
+					intval, uint8_t, DBUS_TYPE_BYTE);
+			break;
+
+		case OFONO_NETMON_INFO_PSC:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "PrimaryScramblingCode",
+					intval, uint16_t, DBUS_TYPE_UINT16);
+			break;
+
+		case OFONO_NETMON_INFO_BER:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "BitErrorRate",
+					intval, uint8_t, DBUS_TYPE_BYTE);
+			break;
+
+		case OFONO_NETMON_INFO_RSSI:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "Strength",
+					intval, uint8_t, DBUS_TYPE_BYTE);
+			break;
+
+		case OFONO_NETMON_INFO_RSCP:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "ReceivedSignalCodePower",
+					intval, uint8_t, DBUS_TYPE_BYTE);
+			break;
+
+		case OFONO_NETMON_INFO_ECN0:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "ReceivedEnergyRatio",
+					intval, uint8_t, DBUS_TYPE_BYTE);
+			break;
+
+		case OFONO_NETMON_INFO_RSRQ:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict,
+					"ReferenceSignalReceivedQuality",
+					intval, uint8_t, DBUS_TYPE_BYTE);
+			break;
+
+		case OFONO_NETMON_INFO_RSRP:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict,
+					"ReferenceSignalReceivedPower",
+					intval, uint8_t, DBUS_TYPE_BYTE);
+			break;
+
+		case OFONO_NETMON_INFO_EARFCN:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "EARFCN",
+					intval, uint16_t, DBUS_TYPE_UINT16);
+			break;
+
+		case OFONO_NETMON_INFO_EBAND:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "EBand",
+					intval, uint8_t, DBUS_TYPE_BYTE);
+			break;
+
+		case OFONO_NETMON_INFO_CQI:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "ChannelQualityIndicator",
+					intval, uint8_t, DBUS_TYPE_BYTE);
+			break;
+
+		case OFONO_NETMON_INFO_PCI:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "PhysicalCellId",
+					intval, uint16_t, DBUS_TYPE_UINT16);
+			break;
+
+		case OFONO_NETMON_INFO_TAC:
+			intval = va_arg(*arglist, int);
+
+			CELL_INFO_DICT_APPEND(dict, "TrackingAreaCode",
+					intval, uint16_t, DBUS_TYPE_UINT16);
+			break;
+
+		case OFONO_NETMON_INFO_SNR:
+			intval = va_arg(*arglist, int);
+
+			ofono_dbus_dict_append(dict, "SingalToNoiseRatio",
+					DBUS_TYPE_INT32, &intval);
+			break;
+
+		case OFONO_NETMON_INFO_INVALID:
+			break;
+		}
+
+		next_info_type = va_arg(*arglist, int);
+	}
+}
+
 void ofono_netmon_serving_cell_notify(struct ofono_netmon *netmon,
 					enum ofono_netmon_cell_type type,
 					int info_type, ...)
@@ -77,11 +233,7 @@ void ofono_netmon_serving_cell_notify(struct ofono_netmon *netmon,
 	DBusMessage *agent_notify = NULL;
 	DBusMessageIter iter;
 	DBusMessageIter dict;
-	enum ofono_netmon_info next_info_type = info_type;
 	const char *technology = cell_type_to_tech_name(type);
-	char *mcc;
-	char *mnc;
-	int intval;
 
 	if (netmon->pending != NULL) {
 		netmon->reply = dbus_message_new_method_return(netmon->pending);
@@ -106,146 +258,7 @@ void ofono_netmon_serving_cell_notify(struct ofono_netmon *netmon,
 	ofono_dbus_dict_append(&dict, "Technology",
 						DBUS_TYPE_STRING, &technology);
 
-	while (next_info_type != OFONO_NETMON_INFO_INVALID) {
-		switch (next_info_type) {
-		case OFONO_NETMON_INFO_MCC:
-			mcc = va_arg(arglist, char *);
-
-			if (mcc && strlen(mcc))
-				ofono_dbus_dict_append(&dict,
-						"MobileCountryCode",
-						DBUS_TYPE_STRING, &mcc);
-			break;
-
-		case OFONO_NETMON_INFO_MNC:
-			mnc = va_arg(arglist, char *);
-
-			if (mnc && strlen(mnc))
-				ofono_dbus_dict_append(&dict,
-						"MobileNetworkCode",
-						DBUS_TYPE_STRING, &mnc);
-			break;
-
-		case OFONO_NETMON_INFO_LAC:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict, "LocationAreaCode",
-					intval, uint16_t, DBUS_TYPE_UINT16);
-			break;
-
-		case OFONO_NETMON_INFO_CI:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict, "CellId",
-					intval, uint32_t, DBUS_TYPE_UINT32);
-			break;
-
-		case OFONO_NETMON_INFO_ARFCN:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict, "ARFCN",
-					intval, uint16_t, DBUS_TYPE_UINT16);
-			break;
-
-		case OFONO_NETMON_INFO_BSIC:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict, "BSIC",
-					intval, uint8_t, DBUS_TYPE_BYTE);
-			break;
-
-		case OFONO_NETMON_INFO_RXLEV:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict, "ReceivedSignalStrength",
-					intval, uint8_t, DBUS_TYPE_BYTE);
-			break;
-
-		case OFONO_NETMON_INFO_TIMING_ADVANCE:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict, "TimingAdvance",
-					intval, uint8_t, DBUS_TYPE_BYTE);
-			break;
-
-		case OFONO_NETMON_INFO_PSC:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict, "PrimaryScramblingCode",
-					intval, uint16_t, DBUS_TYPE_UINT16);
-			break;
-
-		case OFONO_NETMON_INFO_BER:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict, "BitErrorRate",
-					intval, uint8_t, DBUS_TYPE_BYTE);
-			break;
-
-		case OFONO_NETMON_INFO_RSSI:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict, "Strength",
-					intval, uint8_t, DBUS_TYPE_BYTE);
-			break;
-
-		case OFONO_NETMON_INFO_RSCP:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict, "ReceivedSignalCodePower",
-					intval, uint8_t, DBUS_TYPE_BYTE);
-			break;
-
-		case OFONO_NETMON_INFO_ECN0:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict, "ReceivedEnergyRatio",
-					intval, uint8_t, DBUS_TYPE_BYTE);
-			break;
-
-		case OFONO_NETMON_INFO_RSRQ:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict,
-					"ReferenceSignalReceivedQuality",
-					intval, uint8_t, DBUS_TYPE_BYTE);
-			break;
-
-		case OFONO_NETMON_INFO_RSRP:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict,
-					"ReferenceSignalReceivedPower",
-					intval, uint8_t, DBUS_TYPE_BYTE);
-			break;
-
-		case OFONO_NETMON_INFO_EARFCN:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict, "EARFCN",
-					intval, uint16_t, DBUS_TYPE_UINT16);
-			break;
-
-		case OFONO_NETMON_INFO_EBAND:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict, "EBand",
-					intval, uint8_t, DBUS_TYPE_BYTE);
-			break;
-
-		case OFONO_NETMON_INFO_CQI:
-			intval = va_arg(arglist, int);
-
-			CELL_INFO_DICT_APPEND(&dict, "ChannelQualityIndicator",
-					intval, uint8_t, DBUS_TYPE_BYTE);
-			break;
-
-		case OFONO_NETMON_INFO_INVALID:
-			break;
-		}
-
-		next_info_type = va_arg(arglist, int);
-	}
+	netmon_cell_info_dict_append(&dict, &arglist, info_type);
 
 done:
 	va_end(arglist);
@@ -288,7 +301,7 @@ static DBusMessage *netmon_get_serving_cell_info(DBusConnection *conn,
 {
 	struct ofono_netmon *netmon = data;
 
-	if (!netmon->driver && !netmon->driver->request_update)
+	if (!netmon->driver->request_update)
 		return __ofono_error_not_implemented(msg);
 
 	if (netmon->pending)
@@ -353,7 +366,7 @@ static DBusMessage *netmon_register_agent(DBusConnection *conn,
 				DBUS_TYPE_INVALID) == FALSE)
 		return __ofono_error_invalid_args(msg);
 
-	if (!__ofono_dbus_valid_object_path(agent_path))
+	if (!dbus_validate_path(agent_path, NULL))
 		return __ofono_error_invalid_format(msg);
 
 	if (!period)
@@ -403,6 +416,109 @@ static DBusMessage *netmon_unregister_agent(DBusConnection *conn,
 	return dbus_message_new_method_return(msg);
 }
 
+
+void ofono_netmon_neighbouring_cell_notify(struct ofono_netmon *netmon,
+					enum ofono_netmon_cell_type type,
+					int info_type, ...)
+{
+	va_list arglist;
+	DBusMessageIter dict;
+	DBusMessageIter strct;
+	const char *tech = cell_type_to_tech_name(type);
+
+	if (netmon->pending == NULL)
+		return;
+
+	if (!netmon->reply) {
+		netmon->reply = dbus_message_new_method_return(netmon->pending);
+		dbus_message_iter_init_append(netmon->reply, &netmon->iter);
+
+		dbus_message_iter_open_container(&netmon->iter, DBUS_TYPE_ARRAY,
+					DBUS_STRUCT_BEGIN_CHAR_AS_STRING
+					DBUS_TYPE_ARRAY_AS_STRING
+					DBUS_DICT_ENTRY_BEGIN_CHAR_AS_STRING
+					DBUS_TYPE_STRING_AS_STRING
+					DBUS_TYPE_VARIANT_AS_STRING
+					DBUS_DICT_ENTRY_END_CHAR_AS_STRING
+					DBUS_STRUCT_END_CHAR_AS_STRING,
+					&netmon->arr);
+	}
+
+	tech = cell_type_to_tech_name(type);
+
+	dbus_message_iter_open_container(&netmon->arr, DBUS_TYPE_STRUCT,
+						NULL, &strct);
+	dbus_message_iter_open_container(&strct, DBUS_TYPE_ARRAY,
+					OFONO_PROPERTIES_ARRAY_SIGNATURE,
+					&dict);
+
+	va_start(arglist, info_type);
+
+	if (tech == NULL)
+		goto done;
+
+	ofono_dbus_dict_append(&dict, "Technology",
+					DBUS_TYPE_STRING, &tech);
+
+	netmon_cell_info_dict_append(&dict, &arglist, info_type);
+
+done:
+	va_end(arglist);
+
+	dbus_message_iter_close_container(&strct, &dict);
+	dbus_message_iter_close_container(&netmon->arr, &strct);
+}
+
+static void neighbouring_cell_info_callback(const struct ofono_error *error,
+						void *data)
+{
+	struct ofono_netmon *netmon = data;
+	DBusMessage *reply = netmon->reply;
+
+	DBG("");
+
+	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
+		if (reply)
+			dbus_message_unref(reply);
+
+		reply = __ofono_error_failed(netmon->pending);
+        } else if (!reply) {
+		DBusMessageIter iter;
+		DBusMessageIter dict;
+
+		reply = dbus_message_new_method_return(netmon->pending);
+		dbus_message_iter_init_append(reply, &iter);
+		dbus_message_iter_open_container(&iter, DBUS_TYPE_ARRAY,
+					OFONO_PROPERTIES_ARRAY_SIGNATURE,
+					&dict);
+		dbus_message_iter_close_container(&iter, &dict);
+	} else {
+		dbus_message_iter_close_container(&netmon->iter, &netmon->arr);
+	}
+
+	netmon->reply = NULL;
+	__ofono_dbus_pending_reply(&netmon->pending, reply);
+}
+
+static DBusMessage *netmon_get_neighbouring_cell_info(DBusConnection *conn,
+			DBusMessage *msg, void *data)
+{
+	struct ofono_netmon *netmon = data;
+
+	if (!netmon->driver->neighbouring_cell_update)
+		return __ofono_error_not_implemented(msg);
+
+	if (netmon->pending)
+		return __ofono_error_busy(msg);
+
+	netmon->pending = dbus_message_ref(msg);
+
+	netmon->driver->neighbouring_cell_update(netmon,
+				neighbouring_cell_info_callback, netmon);
+
+	return NULL;
+}
+
 static const GDBusMethodTable netmon_methods[] = {
 	{ GDBUS_ASYNC_METHOD("GetServingCellInformation",
 			NULL, GDBUS_ARGS({ "cellinfo", "a{sv}" }),
@@ -413,28 +529,12 @@ static const GDBusMethodTable netmon_methods[] = {
 	{ GDBUS_METHOD("UnregisterAgent",
 			GDBUS_ARGS({ "agent", "o" }), NULL,
 			netmon_unregister_agent) },
+	{ GDBUS_ASYNC_METHOD("GetNeighbouringCellInformation",
+			NULL, GDBUS_ARGS({ "cellinfo", "a(a{sv})" }),
+			netmon_get_neighbouring_cell_info) },
 	{ }
 };
 
-int ofono_netmon_driver_register(const struct ofono_netmon_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	if (d->probe == NULL)
-		return -EINVAL;
-
-	g_drivers = g_slist_prepend(g_drivers, (void *) d);
-
-	return 0;
-}
-
-void ofono_netmon_driver_unregister(const struct ofono_netmon_driver *d)
-{
-	DBG("driver: %p, name: %s", d, d->name);
-
-	g_drivers = g_slist_remove(g_drivers, (void *) d);
-}
-
 static void netmon_unregister(struct ofono_atom *atom)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
@@ -458,38 +558,7 @@ static void netmon_remove(struct ofono_atom *atom)
 	g_free(netmon);
 }
 
-struct ofono_netmon *ofono_netmon_create(struct ofono_modem *modem,
-			unsigned int vendor, const char *driver, void *data)
-{
-	struct ofono_netmon *netmon;
-	GSList *l;
-
-	if (driver == NULL)
-		return NULL;
-
-	netmon = g_try_new0(struct ofono_netmon, 1);
-
-	if (netmon == NULL)
-		return NULL;
-
-	netmon->atom = __ofono_modem_add_atom(modem, OFONO_ATOM_TYPE_NETMON,
-						netmon_remove, netmon);
-
-	for (l = g_drivers; l; l = l->next) {
-		const struct ofono_netmon_driver *drv = l->data;
-
-		if (g_strcmp0(drv->name, driver))
-			continue;
-
-		if (drv->probe(netmon, vendor, data) < 0)
-			continue;
-
-		netmon->driver = drv;
-		break;
-	}
-
-	return netmon;
-}
+OFONO_DEFINE_ATOM_CREATE(netmon, OFONO_ATOM_TYPE_NETMON)
 
 void ofono_netmon_register(struct ofono_netmon *netmon)
 {
